@@ -4,6 +4,9 @@ use crate::event::*;
 mod actor;
 use actor::*;
 
+#[cfg(test)]
+pub use actor::MockKitsuneP2pEventHandler;
+
 /// Spawn a new KitsuneP2p actor.
 pub async fn spawn_kitsune_p2p(
     config: crate::KitsuneP2pConfig,
